/**
 * $Id: Fraction.java,v 1.4 2009/05/01 11:08:19 oboehm Exp $
 *
 * Copyright (c) 2008 by Oliver Boehm
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express orimplied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * (c)reated 19.10.2008 by oliver (ob@oasd.de)
 */
package patterntesting.sample;

import java.math.BigDecimal;

import patterntesting.runtime.annotation.*;

/**
 * This is a litte demo class to show how to use the @UnsupportedOperation
 * annotation.
 * 
 * @author <a href="boehm@javatux.de">oliver</a>
 * @since 19.10.2008
 * @version $Revision: 1.4 $
 */
public class Fraction implements Comparable<Fraction> {
    
    private final int counter;
    private final int denominator;
    
    public Fraction(int counter, int denominator) {
        this.counter = counter;
        if (denominator == 0) {
            throw new IllegalArgumentException("illegal fraction x/0");
        }
        this.denominator = denominator;
    }
    
    public boolean isNegative() {
        return this.toBigDecimal().compareTo(BigDecimal.ZERO) < 0;
    }

    /**
     * This is an example of an automatically generated method by Eclipse.
     * If you extends this class by the Comparable interface but haven't
     * implement the compareTo method you can ask Eclipse to do that by
     * the quick fix "Add unimplemented methods".
     * <br/>
     * The problem with such generated default implementation is that
     * the result may look ok but you can't rely on it. For this reason you
     * should throw an RuntimeException or something else if you have not
     * implement it. This is where the @NotYetImplemented annotation
     * comes in.
     */
    @NotYetImplemented
    public int compareTo(Fraction o) {
        // Auto-generated method stub
        return 0;
    }
    
    /**
     * This is a very simple implementation for toBigDecimal().
     * @return the fraction as decimal
     */
    public BigDecimal toBigDecimal() {
        return new BigDecimal(this.counter / (double) this.denominator);
    }
    
    /**
     * Because of rounding problems this method should be no longer used.
     * So it is now marked as @UnsupportedOperation.
     * <br/>
     * Normally I would mark it also as "@Deprecated". But since it is only
     * an example I abstain from it.
     * 
     * @return
     */
    @UnsupportedOperation
    public double toDouble() {
        return this.counter / (double) this.denominator;
    }

	@Override
	public String toString() {
		return this.counter + "/" + this.denominator;
	}

}

/**
 * $Log: Fraction.java,v $
 * Revision 1.4  2009/05/01 11:08:19  oboehm
 * example for @NotYetImplemented added
 *
 * Revision 1.3  2009/03/16 21:50:45  oboehm
 * example for JUnit anti patterns added
 *
 * Revision 1.2  2009/03/15 21:38:29  oboehm
 * some tests added which shows one JUnit anti pattern
 *
 * Revision 1.1  2008/11/08 18:44:52  oboehm
 * some samples how to use the PatternTesting libs
 *
 * $Source: /cvsroot/patterntesting/PatternTesting08/patterntesting-samples/src/main/java/patterntesting/sample/Fraction.java,v $
 */
