/**
 * $Id: CoffeeAccount.java,v 1.5 2009/02/07 21:40:49 oboehm Exp $
 *
 * Copyright (c) 2008 by Oliver Boehm
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express orimplied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * (c)reated 31.01.2009 by oliver (ob@oasd.de)
 */
package patterntesting.sample.dbc;

import static patterntesting.runtime.dbc.DbC.*;
import patterntesting.runtime.dbc.Contract;

/**
 * This is a simple example application to show the use of DbC (Design by
 * Contract)
 * 
 * @author <a href="boehm@javatux.de">oliver</a>
 * @since 31.01.2009
 * @version $Revision: 1.5 $
 */
public class CoffeeAccount implements Comparable<CoffeeAccount>, Contract {

    private final String name;
    private int balance = 0;
    private static final int PRICE_PER_CUP=25;
    
    /**
     * @see patterntesting.runtime.dbc.Contract#invariant()
     */
    public boolean invariant() {
        return this.balance >= 0;
    }

    public CoffeeAccount(String name) {
        this.name = name;
    }

    public void payIn(int amount) {
        require(amount > 0, "amount must be positive");
        this.balance += amount;
    }

    public void getCupOfCoffee() {
        require(this.balance >= PRICE_PER_CUP, "not enough money");
        this.balance -= PRICE_PER_CUP;
        debitServiceFee();
    }
    
    public void debitServiceFee() {
        this.balance -= 1;
    }

    /**
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    public int compareTo(CoffeeAccount other) {
        return this.name.compareTo(other.name);
    }

    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals(Object other) {
        try {
            return equals((CoffeeAccount) other);
        } catch (ClassCastException e) {
            return false;
        }
    }
    
    public boolean equals(CoffeeAccount other) {
        return this.name.equals(other.name) && (this.balance == other.balance);
    }

    /* (non-Javadoc)
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode() {
        // TODO Auto-generated method stub
        return super.hashCode();
    }

    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return "CoffeeAccount for " + this.name;
    }

    public static void main(String[] args) {
        CoffeeAccount account = new CoffeeAccount("Hugo");
        account.payIn(40);
        account.getCupOfCoffee();
        /** here you will get a ContractViolation */
        account.getCupOfCoffee();
     }

}

/**
 * $Log: CoffeeAccount.java,v $
 * Revision 1.5  2009/02/07 21:40:49  oboehm
 * debitServiceFee() added to show contract violation
 *
 * Revision 1.4  2009/02/03 20:08:30  oboehm
 * DbC support moved from patterntesting-check to patterntesting-rt
 *
 * Revision 1.3  2009/02/03 19:31:15  oboehm
 * invariant() put in front
 *
 * Revision 1.2  2009/02/01 22:18:54  oboehm
 * Collections as example for DbC added
 *
 * Revision 1.1  2009/01/31 21:23:31  oboehm
 * example for DbC added
 *
 * $Source: /cvsroot/patterntesting/PatternTesting08/patterntesting-samples/src/main/java/patterntesting/sample/dbc/CoffeeAccount.java,v $
 */
